@@ -1,5 +1,5 @@
 /*
- * iperf, Copyright (c) 2014-2022, The Regents of the University of
+ * iperf, Copyright (c) 2014-2023, The Regents of the University of
  * California, through Lawrence Berkeley National Laboratory (subject
  * to receipt of any required approvals from the U.S. Dept. of
  * Energy).  All rights reserved.
@@ -51,6 +51,33 @@
 #endif /* TCP_CA_NAME_MAX */
 #endif /* HAVE_TCP_CONGESTION */
 
+void *
+iperf_client_worker_run(void *s) {
+    struct iperf_stream *sp = (struct iperf_stream *) s;
+    struct iperf_test *test = sp->test;
+
+    /* Allow this thread to be cancelled even if it's in a syscall */
+    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
+    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+
+    while (! (test->done) && ! (sp->done)) {
+        if (sp->sender) {
+            if (iperf_send_mt(sp) < 0) {
+                goto cleanup_and_fail;
+            }
+        }
+        else {
+            if (iperf_recv_mt(sp) < 0) {
+                goto cleanup_and_fail;
+            }
+        }
+    }
+    return NULL;
+
+  cleanup_and_fail:
+    return NULL;
+}
+
 int
 iperf_create_streams(struct iperf_test *test, int sender)
 {
@@ -115,12 +142,6 @@ iperf_create_streams(struct iperf_test *test, int sender)
 	}
 #endif /* HAVE_TCP_CONGESTION */
 
-	if (sender)
-	    FD_SET(s, &test->write_set);
-	else
-	    FD_SET(s, &test->read_set);
-	if (s > test->max_fd) test->max_fd = s;
-
         sp = iperf_new_stream(test, s, sender);
         if (!sp)
             return -1;
@@ -519,9 +540,11 @@ iperf_run_client(struct iperf_test * test)
     struct iperf_time last_receive_time;
     struct iperf_time diff_time;
     struct timeval used_timeout;
+    iperf_size_t last_receive_blocks;
     int64_t t_usecs;
     int64_t timeout_us;
     int64_t rcv_timeout_us;
+    int i_errno_save;
 
     if (NULL == test)
     {
@@ -562,6 +585,9 @@ iperf_run_client(struct iperf_test * test)
     else
         rcv_timeout_us = 0;
 
+    iperf_time_now(&last_receive_time); // Initialize last time something was received
+    last_receive_blocks = 0;
+
     startup = 1;
     while (test->state != IPERF_DONE) {
 	memcpy(&read_set, &test->read_set, sizeof(fd_set));
@@ -577,6 +603,10 @@ iperf_run_client(struct iperf_test * test)
                 used_timeout.tv_usec = timeout->tv_usec;
                 timeout_us = (timeout->tv_sec * SEC_TO_US) + timeout->tv_usec;
             }
+            /* Cap the maximum select timeout at 1 second */
+            if (timeout_us > SEC_TO_US) {
+                timeout_us = SEC_TO_US;
+            }
             if (timeout_us < 0 || timeout_us > rcv_timeout_us) {
                 used_timeout.tv_sec = test->settings->rcv_timeout.secs;
                 used_timeout.tv_usec = test->settings->rcv_timeout.usecs;
@@ -589,23 +619,32 @@ iperf_run_client(struct iperf_test * test)
   	    i_errno = IESELECT;
 	    goto cleanup_and_fail;
         } else if (result == 0 && test->state == TEST_RUNNING && rcv_timeout_us > 0) {
-            // If nothing was received in non-reverse running state then probably something got stack -
-            // either client, server or network, and test should be terminated.
+            /*
+             * If nothing was received in non-reverse running state
+             * then probably something got stuck - either client,
+             * server or network, and test should be terminated./
+             */
             iperf_time_now(&now);
             if (iperf_time_diff(&now, &last_receive_time, &diff_time) == 0) {
                 t_usecs = iperf_time_in_usecs(&diff_time);
                 if (t_usecs > rcv_timeout_us) {
-                    i_errno = IENOMSG;
-                    goto cleanup_and_fail;
+                    /* Idle timeout if no new blocks received */
+                    if (test->blocks_received == last_receive_blocks) {
+                        i_errno = IENOMSG;
+                        goto cleanup_and_fail;
+                    }
                 }
 
             }
         }
 
+        /* See if the test is making progress */
+        if (test->blocks_received > last_receive_blocks) {
+            last_receive_blocks = test->blocks_received;
+            last_receive_time = now;
+        }
+
 	if (result > 0) {
-            if (rcv_timeout_us > 0) {
-                iperf_time_now(&last_receive_time);
-            }
 	    if (FD_ISSET(test->ctrl_sck, &read_set)) {
  	        if (iperf_handle_message_client(test) < 0) {
 		    goto cleanup_and_fail;
@@ -620,32 +659,32 @@ iperf_run_client(struct iperf_test * test)
 	    if (startup) {
 	        startup = 0;
 
-		// Set non-blocking for non-UDP tests
-		if (test->protocol->id != Pudp) {
-		    SLIST_FOREACH(sp, &test->streams, streams) {
-			setnonblocking(sp->socket, 1);
-		    }
-		}
+                /* Create and spin up threads */
+                pthread_attr_t attr;
+                if (pthread_attr_init(&attr) != 0) {
+                    i_errno = IEPTHREADATTRINIT;
+                    goto cleanup_and_fail;
+                }
+
+                SLIST_FOREACH(sp, &test->streams, streams) {
+                    if (pthread_create(&(sp->thr), &attr, &iperf_client_worker_run, sp) != 0) {
+                        i_errno = IEPTHREADCREATE;
+                        goto cleanup_and_fail;
+                    }
+                    if (test->debug_level >= DEBUG_LEVEL_INFO) {
+                        iperf_printf(test, "Thread FD %d created\n", sp->socket);
+                    }
+                }
+                if (test->debug_level >= DEBUG_LEVEL_INFO) {
+                    iperf_printf(test, "All threads created\n");
+                }
+                if (pthread_attr_destroy(&attr) != 0) {
+                    i_errno = IEPTHREADATTRDESTROY;
+                    goto cleanup_and_fail;
+                }
+
 	    }
 
-
-	    if (test->mode == BIDIRECTIONAL)
-	    {
-                if (iperf_send(test, &write_set) < 0)
-                    goto cleanup_and_fail;
-                if (iperf_recv(test, &read_set) < 0)
-                    goto cleanup_and_fail;
-	    } else if (test->mode == SENDER) {
-                // Regular mode. Client sends.
-                if (iperf_send(test, &write_set) < 0)
-                    goto cleanup_and_fail;
-	    } else {
-                // Reverse mode. Client receives.
-                if (iperf_recv(test, &read_set) < 0)
-                    goto cleanup_and_fail;
-	    }
-
-
             /* Run the timers. */
             iperf_time_now(&now);
             tmr_run(&now);
@@ -665,12 +704,33 @@ iperf_run_client(struct iperf_test * test)
 	         (test->settings->blocks != 0 && (test->blocks_sent >= test->settings->blocks ||
 						  test->blocks_received >= test->settings->blocks)))) {
 
-		// Unset non-blocking for non-UDP tests
-		if (test->protocol->id != Pudp) {
-		    SLIST_FOREACH(sp, &test->streams, streams) {
-			setnonblocking(sp->socket, 0);
-		    }
-		}
+                /* Cancel outstanding sender threads */
+                SLIST_FOREACH(sp, &test->streams, streams) {
+                    if (sp->sender) {
+                        int rc;
+                        sp->done = 1;
+                        rc = pthread_cancel(sp->thr);
+                        if (rc != 0 && rc != ESRCH) {
+                            i_errno = IEPTHREADCANCEL;
+                            errno = rc;
+                            iperf_err(test, "sender cancel in pthread_cancel - %s", iperf_strerror(i_errno));
+                            goto cleanup_and_fail;
+                        }
+                        rc = pthread_join(sp->thr, NULL);
+                        if (rc != 0 && rc != ESRCH) {
+                            i_errno = IEPTHREADJOIN;
+                            errno = rc;
+                            iperf_err(test, "sender cancel in pthread_join - %s", iperf_strerror(i_errno));
+                            goto cleanup_and_fail;
+                        }
+                        if (test->debug_level >= DEBUG_LEVEL_INFO) {
+                            iperf_printf(test, "Thread FD %d stopped\n", sp->socket);
+                        }
+                    }
+                }
+                if (test->debug_level >= DEBUG_LEVEL_INFO) {
+                    iperf_printf(test, "Sender threads stopped\n");
+                }
 
 		/* Yes, done!  Send TEST_END. */
 		test->done = 1;
@@ -680,15 +740,34 @@ iperf_run_client(struct iperf_test * test)
                     goto cleanup_and_fail;
 	    }
 	}
-	// If we're in reverse mode, continue draining the data
-	// connection(s) even if test is over.  This prevents a
-	// deadlock where the server side fills up its pipe(s)
-	// and gets blocked, so it can't receive state changes
-	// from the client side.
-	else if (test->mode == RECEIVER && test->state == TEST_END) {
-	    if (iperf_recv(test, &read_set) < 0)
-		goto cleanup_and_fail;
-	}
+    }
+
+    /* Cancel outstanding receiver threads */
+    SLIST_FOREACH(sp, &test->streams, streams) {
+        if (!sp->sender) {
+            int rc;
+            sp->done = 1;
+            rc = pthread_cancel(sp->thr);
+            if (rc != 0 && rc != ESRCH) {
+                i_errno = IEPTHREADCANCEL;
+                errno = rc;
+                iperf_err(test, "receiver cancel in pthread_cancel - %s", iperf_strerror(i_errno));
+                goto cleanup_and_fail;
+            }
+            rc = pthread_join(sp->thr, NULL);
+            if (rc != 0 && rc != ESRCH) {
+                i_errno = IEPTHREADJOIN;
+                errno = rc;
+                iperf_err(test, "receiver cancel in pthread_join - %s", iperf_strerror(i_errno));
+                goto cleanup_and_fail;
+            }
+            if (test->debug_level >= DEBUG_LEVEL_INFO) {
+                iperf_printf(test, "Thread FD %d stopped\n", sp->socket);
+            }
+        }
+    }
+    if (test->debug_level >= DEBUG_LEVEL_INFO) {
+        iperf_printf(test, "Receiver threads stopped\n");
     }
 
     if (test->json_output) {
@@ -704,6 +783,32 @@ iperf_run_client(struct iperf_test * test)
     return 0;
 
   cleanup_and_fail:
+    /* Cancel all outstanding threads */
+    i_errno_save = i_errno;
+    SLIST_FOREACH(sp, &test->streams, streams) {
+        sp->done = 1;
+        int rc;
+        rc = pthread_cancel(sp->thr);
+        if (rc != 0 && rc != ESRCH) {
+            i_errno = IEPTHREADCANCEL;
+            errno = rc;
+            iperf_err(test, "cleanup_and_fail in pthread_cancel - %s", iperf_strerror(i_errno));
+        }
+        rc = pthread_join(sp->thr, NULL); 
+        if (rc != 0 && rc != ESRCH) {
+            i_errno = IEPTHREADJOIN;
+            errno = rc;
+            iperf_err(test, "cleanup_and_fail in pthread_join - %s", iperf_strerror(i_errno));
+        }
+        if (test->debug_level >= DEBUG_LEVEL_INFO) {
+            iperf_printf(test, "Thread FD %d stopped\n", sp->socket);
+        }
+    }
+    if (test->debug_level >= DEBUG_LEVEL_INFO) {
+        iperf_printf(test, "All threads stopped\n");
+    }
+    i_errno = i_errno_save;
+
     iperf_client_end(test);
     if (test->json_output) {
         cJSON_AddStringToObject(test->json_top, "error", iperf_strerror(i_errno));
